
function [w, tri, poslist, I] = FWT2_2GLS(x,y,f,M)

% usage: [w, tri, poslist, I] = FWT2_2GLS(x,y,f,M);
% input:
% ------
% x,y : equally sized vectors with grid point locations
% f : function values on grid points: length(f) = length(x)
%     if f is a square matrix, the transform is performed on its columns
%     use Wt = FWT2_2GLS(x,y,eye(length(x)),M);
%     to obtain the forward wavelet trasnform matrix,
%     type w = Wt*column(f);
%     to obtain the coefficients through matrix-vector multiplication (this is
%     obviously an expensive route to obtain wavelet coefficients)
% M : number of steps/scales to be transformed (must be smaller than
% length(f)-1)
%
% output:
% -------
% w : wavelet coefficients
% tri: multiscale mesh
% poslist: list of grid point locations, removed during the transform
% I : integral of coarsest scaling functions associated with grid points

fsize = size(f);
x = column(x);
y = column(y);
f = column(f);
M = min(M,length(x)-2);
[tri B] = inittri(x,y);
tri = initmultitri(tri);
I = initint2(tri,x,y);
left = (1:length(x));
% left contains indices of remaining vertices
poslist = zeros(M,1);
% poslist keeps track of the order in which the vertices are being removed
w = f;
% initialisation of wavelet coefficients
for j = 1:M
   [Imin v0] = min(abs(I(left)));
   v0 = left(v0);
   left = left(find(left~=v0));
      % this action removes v0 from remaining vertices
   poslist(j) = v0;
   % [tri, w, I] = onestepconvextriLS(tri,x,y,v0,j,w,I);
   % [tri, w, I] = onestepLS(tri,x,y,v0,j,w,I,B);
   [tri, w, I, B] = onestepLS(tri,x,y,v0,j,w,I, B);
end
w = reshape(w,fsize);
