
function addtri = bridgegaps(tri,v0,points,gaps,x,y);

% usage: addtri = bridgegaps(tri,v0,points,gaps,x,y);
% finds triangles to add to a triangulation such that no vertex constitutes a
% single connection between two parts of the triangulation
% !!! points(1) and points(last) must not be gap points!!!
%
% See also: findgapsonboundary.m

tri = tri(:,1:3);
addtri = zeros(0,3);
n = length(points);
% the closed loop [points v0] is always convex, so anglesboundary runs here for
% sure.

% remove end points from list of gap points
nogaps = find(ismember(gaps,points([1 n])));
gaps(nogaps) = 0;
gaps = gaps(find(gaps));

% gaps may contain more than one gap, so first, we identify a cluster of points
% in "points" that constitute one gap.

E = listedges(tri);
while ~isempty(gaps),
   % So, find the first gap in list of gaps
   gap = gaps(1);
   i = find(points==gap(1));
   pp = points(i:i+1);
   while i<n-1 & ~any(all(ismember(E,pp),2)),
      i = i+1;
      gap = [gap points(i)];
      pp = points(i:i+1);
   end
   i = find(points==gap(1));
   pp = points(i-1:i);
   while i>2 & ~any(all(ismember(E,pp),2)),
      i = i-1;
      gap = [points(i) gap];
      pp = points(i-1:i);
   end
   gaps = setdiff(gaps,gap);

   % Now, identify first and last point in gap
   g = length(gap);
   p = gap(1);
   q = gap(g);
   i = find(points==p);
   j = i+g-1;

   % Find triangles containing those gap end points
   [np Tp] = findneighbours(p,tri);
   [nq Tq] = findneighbours(q,tri);
   p1 = points(i-1);
   q1 = points(j+1);

   % now walk through triangulations Tp and Tq to find neighbouring point(s), 
   % on the boundary (on each side of the gap). These points (if and where
   % they exist) are candidates to start the construction of the bridge.
   % first find p1 and q1, then start from there looking for a possibly
   % different p2 and q2. Scanning for p1 and q1 starts in points(i-1) and
   % points(j+1) respectively. This choice guarantees that we end up on
   % different sides of the gap if p=q (if the gap has only one vertex)
   tp = find(sum(ismember(Tp,p1),2));
   p1prev = p;
   while length(tp) == 2,
      p1prev = [p1prev p1];
      p1 = setdiff(unique(Tp(tp,1:3)),p1prev);
      p1 = p1(1);
      tp = find(sum(ismember(Tp,p1),2));
   end

   tq = find(sum(ismember(Tq,q1),2));
   q1prev = q;
   while length(tq) == 2,
      q1prev = [q1prev q1];
      q1 = setdiff(unique(Tq(tq,1:3)),q1prev);
      q1 = q1(1);
      tq = find(sum(ismember(Tq,q1),2));
   end

   p2 = p1;
   q2 = q1;
   tp = find(sum(ismember(Tp,p2),2));
   while ~isempty(tp),
      p2 = setdiff(Tp(tp,1:3),[p2 p]);
      Tp(tp,1:3) = [0 0 0];
      tp = find(sum(ismember(Tp,p2),2));
   end
   tq = find(sum(ismember(Tq,q2),2));
   while ~isempty(tq),
      q2 = setdiff(Tq(tq,1:3),[q2 q]);
      Tq(tq,1:3) = [0 0 0];
      tq = find(sum(ismember(Tq,q2),2));
   end
   % Now we have two possible end points of the bridge on each side of the gap
   % (those two points may coincide). We now decide which point to use on each
   % side.
   if g == 1, % i.e., p = q
              % In this case add the triangle p1pq1 or p2pq2. Before deciding
              % which one, make sure that p1 and q1 can be connected without
              % intersecting the existing triangulation (same for p2q2)
      alfa(1) = abs(anglespolyline([p1 p q1],x,y));
      alfa(2) = abs(anglespolyline([p1 p q2],x,y));
      alfa(3) = abs(anglespolyline([p2 p q1],x,y));
      alfa(4) = abs(anglespolyline([p2 p q2],x,y));
      [minalfa i] = min(alfa);
      switch i
      case 1
         bridge = [p1 p q1];
      case 2
         bridge = [p1 p q2];
      case 3
         bridge = [p2 p q1];
      case 4
         bridge = [p2 p q2];
      end
   else
      % decide upon end point of bridge
      b1 = anglesboundary([points(j-1) q q1],x,y);
      b2 = anglesboundary([points(j-1) q q2],x,y);
      b1 = b1(2);
      b2 = b2(2);
      if b2 < b1,
         q1 = q2;
         b1 = b2;
      end
      % decide upon starting point of bridge
      a1 = anglesboundary([p1 p points(i+1)],x,y);
      a2 = anglesboundary([p2 p points(i+1)],x,y);
      a1 = a1(2);
      a2 = a2(2);
      if a2 < a1,
         p1 = p2;
         a1 = a2;
      end
      % now construct the bridge
      p1gapq1 = [p1 gap q1];
      angles = anglespolyline(p1gapq1,x,y);
      nn = length(p1gapq1);
      bridge = zeros(0,3);
      cluster = p1gapq1(1:3);
      for j = 3:nn-1,
         if angles(j-1) < pi,
            cluster = [cluster p1gapq1(j+1)];
         else
            clustertri = inittri(x(cluster),y(cluster));
            clustertri = cluster(clustertri);
            bridge = [bridge' clustertri']';
            cluster = p1gapq1(j-1:j+1);
            angles = 2*pi - angles;
         end
      end
      clustertri = inittri(x(cluster),y(cluster));
      clustertri = cluster(clustertri);
      bridge = [bridge' clustertri']';
   end
   addtri = [addtri' bridge']';
   tri = [tri' bridge']';
end
if ~isempty(addtri),
   % disp('Warning: reconnecting procedure activated (bridgegaps.m)');
   bridgegapsactivated = 1;
end
