
function [tri, E] = closeboundary(points,v0,x,y);

% usage: tri = closeboundary(points,v0,x,y);
% closes boundary between points(1) and points(last)
% (x0,y0) = (v(x0),v(y0)) indicates on which side to close
% edges between successive points must have no intersections

tri = zeros(0,3);
E = zeros(0,2);
n = length(points);

alfa = anglesboundary([points v0],x,y);
if alfa(n+1) > pi,
   % if the removed point is inside the convex hull of the others, then closure
   % of boundary is trivial (and the nontrivial approach below would possibly
   % crash on overlapping triangles).
   shortcut = [points(1) points(n)];
else
   % now find boundary line connecting points(1) and points(n) with no convex
   % corners in between, i.e., the shortest shortcut of points
   shortcut = points;
   convexangles = 1+find(alfa(2:n-1)<pi);
   while ~isempty(convexangles),
      p = convexangles(1);
      shortcut(p) = 0;
      shortcut = shortcut(find(shortcut)); % this removes zeros
      alfa = anglesboundary([shortcut v0],x,y);
      n = n-1;
      convexangles = 1+find(alfa(2:n-1)<pi);
   end
end

% now triangulate all pieces of the polyline of points between two successive
% points in shortcut

for s = 2:length(shortcut);
   p = find(points==shortcut(s-1));
   pp = find(points==shortcut(s));
   tt = triangulate(points(p:pp),x,y);
   % now remove narrow triangles near boundary (cfr. inittri.m)
   tt = removenarrowtrinearboundary(tt,x,y,shortcut(s-1:s));
   tri = [tri' tt']';
end

if ~isempty(tri),
   E = listedges(tri);
end
