function gappoints = findgapsonboundary(B,points,tri);

% usage: gappoints = findgapsonboundary(B,points,tri);
% B is boundary of connected triangulation, before last step in transform.
% gap is set of points that should be connected but which may be disconnected
% after the last step.
% tri is the local retriangulation in the last step (closeboundary.m); 
% or the triangulation after removing v0 from the grid and adding the result
% from closeboundary.m
% Since this resulting triangulation may be non-connected (that's why we use
% bridgegaps.m), we cannot use the function boundary.m to find that boundary,
% and so we work with B, the boundary of the triangulation before removing v0.
%
% This function only provides a list of gap points. It does _not_ tell which of
% these points constitute one gap. 

if isempty(tri),
   tri = zeros(0,3);
end
n = length(points);
gappoint = ismember(points,B);
gappoint(1) = 0;
gappoint(n) = 0;
% find all triangles within set of points (i.e., newly added triangles)
tt = find(all(ismember(tri(:,1:3),points),2));
% these new triangles constitute bridges, now find the bridged vertices. Those
% vertices are no gap points. !!We use the fact that points is an ordered set!!
% The following short for-loop expression only works in Eindhoven, not in
% Leuven:
% for t = row(tt),
for r = 1:length(tt),
   t = tt(r);
   ijk = sort(find(ismember(points,tri(t,1:3))));
   j = ijk(2);
   gappoint(j) = 0;
end
gappoints = points(find(gappoint));
