
function [w, If] = invstepLS(tri,x,y,v0,j,w0,Ic)

% usage: [w, If] = invstepLS(tri,x,y,v0,j,w0,Ic)
% one step of inverse transform
% v0 = vertex to be added
% w0 = wavelet coefficients at input
% j = level
% if j = 1, then w contains observations
% Ic: scaling function integrals at coarse scale
% If: scaling function integrals at fine scale

N = length(x);
% first look up which points are neighbors of the added point.
tri0j = leveltri(tri,j-1);
[points oldtri] = findneighbours(v0,tri0j);

w = w0;
weights = filtercoefLS(x(v0),y(v0),x(points),y(points));

%%%UPDATE%%%
If = Ic;
If(points) = Ic(points) - weights*If(v0)/sum(weights);
% note that If(v0) = Ic(v0) contains correct value: see note in onestep.m
w(points,:) = w(points,:) - ...
              column(Ic(points))*w(v0,:)*If(v0)/sum(Ic(points).^2);

%%%PREDICTION%%%

w(v0,:) = w0(v0,:) + row(weights)*w(points,:)/sum(weights);
