function [a,b] = linerrinvar(X,Y);

% usage: [a,b] = linerrinvar(X,Y);
% Errors-in-Variables regression (Total Least Squares) for data (X,Y)
% a and b are intercept and slope of fitting line: y = a + bx
% if b = Inf, then we have a vertical line. In that case, the output for a
% is exceptionally the value in the expression of the line x = a;
%
% reference:
% http://mathworld.wolfram.com/LeastSquaresFittingPerpendicularOffsets.html

x = mean(X);
y = mean(Y);
xx = sum(X.^2);
yy = sum(Y.^2);
xy = sum(X.*Y);
n = length(X);

if abs(n*x*y - xy) < eps, % i.e., regression line will be either horizontal
                          % or vertical
   R1 = sum((Y-mean(Y)).^2);
   R2 = sum((X-mean(X)).^2);
   if R1 < R2,
      a = mean(Y);
      b = 0;
   else
      a = mean(X);
      b = Inf;
   end
else
   B = 0.5 * (yy - xx - n*y^2 + n*x^2)/(n*x*y - xy);
   b1 = -B + sqrt(B^2 + 1);
   b2 = -B - sqrt(B^2 + 1);
   a1 = y - b1*x;
   a2 = y - b2*x;

   R1 = sum((Y - a1 - b1*X).^2) / (1 + b1^2);
   R2 = sum((Y - a2 - b2*X).^2) / (1 + b2^2);

   if R1 < R2,
      a = a1;
      b = b1;
   else
      a = a2;
      b = b2;
   end
end
% if isinf(b),
%    [a,o] = linLS1d(Y,X);
% end
