
function [tri, w, Ic, B] = onestepLS(tri0,x,y,v0,j,w0,If,B)

% usage: [tri, w, Ic, B] = onestepLS(tri0,x,y,v0,j,w0,If,B)
% extension of subsample
% v0 = vertex to be removed
% w0 = wavelet coefficients at input
% j = level
% if j = 1, then w0 contains observations
% Ic: scaling function integrals at coarse scale
% If: scaling function integrals at fine scale

% next four lines can be activated for testing
% global TT
% TT = tri0;
% j
% v0
global tri0j
global tri00
global tri1
global tri0g
global Bglobal

N = length(x);
tri0j = leveltri(tri0,j-1);
[points oldtri] = findneighbours(v0,tri0j);
noldtri = size(oldtri,1);
intri = (tri0(:,1:3) == v0);
% (find positions of v0 in list of triangles)
s = find(sum(intri,2).*(tri0(:,5)==0));
% it holds that: oldtri = tri0(s,1:3);
% oldtri contains triangles to be removed

% points is a vector of indices corresponding to neighbouring vertices of v0
% This set is retriangulated. Pay attention to the order in points: it
% matters.
% If v0 is a boundary vertex, points(1) and points(last) must also be boundary
% vertices. If there are more boundary vertices, points(1) and points(last)
% cannot be neighbours.
if (length(points) <= 2),
   % We are on the boundary, but as only 2 points are neighbours of the removed
   % point, removal can proceed without retriangulation
   tri1 = zeros(0,3);
   B = B(find(B~=v0));
   % the above line is a simplified version of the more general statement
   % below:
   %  [tri0j tri00] = removevertexfromtri(v0,tri0j);
   %  B = updateboundary(B,tri0j,tri00,tri1);
elseif (length(points) > noldtri)  %VOORZICHTIG... vroeger stond hier EE
   % i.e.; eliminated vertex was on boundary ->
   % special precautions are necessary
   tri1a = closeboundary(points,v0,x,y);
   % tri0j = leveltri(tri0,j-1); %% done before
   if nargin < 8, B = boundary(tri0j); end
   gaps = findgapsonboundary(B,points,tri1a);
   % Now we update tri0j to make it ready as input for bridgegaps
   [tri0j tri00] = removevertexfromtri(v0,tri0j);
   tri0j = [tri0j' tri1a']';
   tri1b = bridgegaps(tri0j,v0,points,gaps,x,y);
   tri1 = [tri1a' tri1b']';
   % Now we re-compute tri0j and tri00 for the input of updateboundary
   [tri0j tri00] = removevertexfromtri(v0,leveltri(tri0,j-1));
   B = updateboundary(B,tri0j,tri00,tri1);
else
   tri1 = triangulate(points,x,y);
end
m = size(tri1,1);
if m == 0,
   tri1 = zeros(m,5);
else
   tri1 = [tri1 j*ones(m,1) zeros(m,1)];
end
tri0(s,5) = j*ones(noldtri,1);
% Now, extend multiscale triangulation with new triangles and with
% removal information for old ones.
tri = [tri0' tri1']';

w = w0;

%%% COMPUTE FILTER COEFF. FROM POINT POSITIONS %%%

weights = filtercoefLS(x(v0),y(v0),x(points),y(points));
% these weights are the prediction coefficients

%%%PREDICTION%%%
%% properties of prediction operator depend on choice of filter coefficients
%% edit filtercoef.m to make appropriate choice.

w(v0,:) = w0(v0,:) - row(weights)*w0(points,:)/sum(weights);

%%%UPDATE%%%
%% update is minimum norm filter with one (dual) vanishing moment,
%% corresponding to given prediction.

% first compute integrals of scaling functions at new, "coarse" scale.

Ic = If;
Ic(points) = If(points) + weights*If(v0)/sum(weights);
w(points,:) = w(points,:) + ...
              column(Ic(points))*w(v0,:)*If(v0)/sum(Ic(points).^2);

% note that Ic(v0) is still If(v0): this is of course not the wavelet function
% integral (so, strictly spoken, this value is incorrect). Nevertheless,
% since we know what the exact value is (namely:0), we can use this memory 
% cell to stock a value which we can reuse in the inverse transform.
