
function [Wtstep, Ic] = onestepmatrixLS(tri,x,y,v0,j,If)

% usage: [Wtstep, Ic] = onestepmatrixLS(tri,x,y,v0,j,If);
% or   : Wstep = onestepmatrix(tri,x,y,v0,j,If);
% constructs (sparse) matrix for j-th step of a *given* wavelet transform.
% i.e.: the multiresolution triangularization is supposed to be known, as well
% as the order in which data points are considered. In practice this requires
% a wavelet transform (of arbitrary data) up to scale j at least.
% see FWT2_2G.m for details on this transform

N = length(x);
Wpred = sparse(eye(N));
Wupd = Wpred;
% first look up which points are neighbors of the removed point.
% this can be found in the multiresolution triangularization, as computed from
% the wavelet transform in FWT2_2G.m and onestep.m
s = find(tri(:,5) == j);
oldtri = tri(s,1:3);
% oldtri contains triangles, removed in step j
points = findneighbours(v0,oldtri);

weights = filtercoefLS(x(v0),y(v0),x(points),y(points));

%%%PREDICTION%%%

Wpred(v0,points) = - row(weights)/sum(weights);

%%%UPDATE%%%
Ic = If;
Ic(points) = If(points) + weights*If(v0)/sum(weights);
Wupd(points,v0) = column(Ic(points))*If(v0)/sum(Ic(points).^2);

%%%ALL TOGHETHER%%%

% Wpred = sparse(Wpred);
% Wupd = sparse(Wupd);
Wtstep = Wupd*Wpred;
