
function [tri, E] = triangulate(points,x,y);

% usage: [tri, E] = triangulate(points,x,y);
%
% triangulation of a set of points, where all points are on the boundary
% (otherwise, use delaunay, inittri, or cstrdelaunay)
%
% The boundary in "points" is assumed to be a set of edges that can be part of
% a Delaunay triangulation. This is the case when that boundary is the boundary
% of a hole that results from taking away a single non-boundary point from an
% existing Delaunay-triangulation. A counterexample is: boundary = (1:13) and
% x = [1 4 1.1 5 9 7 9 6 5 3 1 2 0];
% y = [9 3 9 9 7 5 4 1 0 2 3 5 8];
%     (MJ 2/8/4)
% In such cases, use triangulateQD.m
%
% new, constructive version.
% The old, destructive version (using delaunay, followed by exception handling)
% is now in triangulateOLD.m
%
% See also: triangulateQD.m, inittri.m, cstrdelaunay.m

npoints = length(points);
if npoints < 3,
   tri = zeros(0,3);
   E = zeros(0,2);
else
   tri = bndrycstrbndrydelaunay1(points,x,y);
   E = listedges(tri);
end
