
function B  = updateboundary(B,tri,tri0,tri1);

% usage: B = updateboundary(B,tri,tri0,tri1);
% This function computes the boundary of tri after tri0 is removed from it and
% tri1 is added to it. It is equivalent to:
%
% remove_tri0_from_tri (optional; if this has already beendone, obviously,
%                       nothing changes)
% B = boundary([tri tri1])
% but the implementation is faster.
% This function assumes that tri0 has already been removed from tri, but the
% boundary has not been recomputed yet.

% First make sure all triangles from tri0 are removed from tri
tri = tri(:,1:3);
tri0 = tri0(:,1:3);
tri = sort(tri,2);
tri0 = sort(tri0,2);
tri0intri = ismember(tri,tri0,'rows');
tri = tri(~tri0intri,1:3);

E0 = sort(listedges(tri0),2);
E1 = sort(listedges(tri1),2);

nB = length(B);

if B(1) ~= B(nB), nB = nB+1; B = [B B(1)]; end
EB = sort([B(1:nB-1)' B(2:nB)'],2);
eb0 = ismember(EB,E0,'rows');
eb1 = ismember(EB,E1,'rows');
% for e = 1:size(E0,1), eb0 = [eb0 find(EB(:,1)==E0(e,1) & EB(:,2)==E0(e,2))]; end
eb01 = eb0 | eb1;
% eb01 is true if first point of corresponding polyline segments in B is to be 
% removed


tri = [tri; tri1];
% following 3 steps from function boundary.m (the time consuming step in
% boundary.m is the 4th step: linkedges. We can do faster if we only have to
% repair a few missing links)
[E etri] = listedges(tri);
e = find(~etri(:,2));
Ebound = E(e,:);

if all(eb01), 
   B1 = linkedges(Ebound);
else
   nB = nB-1;
   B = B(1:nB);
   eb01 = eb01(1:nB);
   startpoint = find(~eb01);
   startpoint = startpoint(1);
   B = shift(B,startpoint-1);
   eb01 = shift(eb01,startpoint-1);
   eb01 = [row(eb01) eb01(1)]; 
   B1(1:2) = B(1:2);
   nB1 = 2;
   b = 2;
   while b > 1, % if b == 1, we arrived at the initial point, so we have closed
                % the loop
      while ~eb01(b) & b < nB,
         b = b+1;
         B1 = [B1 B(b)];
         nB1 = nB1+1;
      end
      newpoint = 0;
      while ~ismember(newpoint,B),
         e = find(any(Ebound==B1(nB1),2));
         if length(e) ~= 2, error('Could not construct new boundary'), end
         if ismember(B1(nB1-1),Ebound(e(1),1:2)), e = e(2); else e = e(1); end
         newpoint = setdiff(Ebound(e,1:2),B1(nB1));
         B1 = [B1 newpoint];
         nB1 = nB1+1;
      end
      b = max(find(B==newpoint)); % max because B(1) == B(nB), in that case we
                                  % need b = nB;
   end
end

B = B1;
